##################################################
## Inserts and converts the topography regions of
## the GSM output topography file into Loveclim,
## do the regridding (weighted average method),
## and saves and renames output to look like
## exactly as the "berg.nc" file needed by 
## Loveclim.
## Also creates the "icemask.nc" file required by
## Loveclim.
## Icemask is defined as cells ice cover >= 30%
##
## exported items:
##                ${execLove}
##                ${execGSM}
##                ${upscl}: Upscaling method:
##                             1. Simple Average
##                             2. Envelope Method
##                             3. Silhouette Method
## 

##############################################################
### ERROR HANDLING
##  err: Determines important errors from unimportant errors
##       Set it to a non-zero value before commands to stop
##        if they fail, and zero after them again
##       e.g.: err=1
##             command; err=0

err=0
error() {
    if [ ${err} -ne '0' ]; then
        echo "file $(basename $0) failed at line $1"
        exit 1
    fi
}
trap 'error ${LINENO}' ERR
##############################################################

log()
{
    eval $@
    if [[ ${debug} == 1 ]]; then
        echo $@
    fi
}


cat<<EOF
##############################################################
### cnvGSMLoveEns.sh

EOF


##################################################
### Name of Loveclim and GSM topography files

TopoLove=berg.nc
IceLove=icemask.nc

cd ${execGSM}
if [[ -e ${execLove}/LOVECLIM/V1.3/scenario/Default/orgicemask.nc ]]; then
#    ln -s ${execLove}/LOVECLIM/V1.3/scenario/Default/orgberg.nc ${TopoLove}
    ln -s ${execLove}/LOVECLIM/V1.3/scenario/Default/orgicemask.nc ${IceLove}
else
#    ln -s ${cplCndDir}/topo/${berg} ${TopoLove}
    ln -s ${cplCndDir}/topo/${icemask} ${IceLove}
fi

cp topoGSMres.org.nc topoGSMres.nc

cat <<EOF > ffConvTopo.jnl
DEFINE AXIS/T=${nextOrby}:${nextOrby}:1 time

!! Low Res GSM
! DEFINE AXIS/X=172.5W:42.5W:1/UNIT=DEGREE xlon
! DEFINE AXIS/Y=34.75N:84.75N:0.5/UNIT=DEGREE ylat
! DEFINE GRID/X=xlon/Y=ylat/T=time na105

! DEFINE AXIS/X=12.5W:119.5E:1/UNIT=DEGREE/mod xlone
! DEFINE AXIS/Y=47.25N:83.25N:0.5/UNIT=DEGREE ylate
! DEFINE GRID/X=xlone/Y=ylate/T=time ea105

! DEFINE axis/modulo/x=0.5:359.5:1 xfine
! DEFINE axis/y=-89.75:89.75:0.5 yfine
! DEFINE grid/x=xfine/y=yfine grfine

! DEFINE axis/modulo/x=0.5:359.5:1 xfine
! DEFINE axis/y=-89.75:89.75:0.5 yfine
! DEFINE grid/x=xfine/y=yfine grfine


!! High Res GSM
DEFINE AXIS/X=171.75W:4.75W:0.5/UNIT=DEGREE xlon
DEFINE AXIS/Y=35.875N:84.875N:0.25/UNIT=DEGREE ylat
DEFINE GRID/X=xlon/Y=ylat/T=time na105

DEFINE AXIS/modulo/X=12.75W:119.25E:0.5/UNIT=DEGREE xlone
DEFINE AXIS/Y=48.125N:83.125N:0.25/UNIT=DEGREE ylate
DEFINE GRID/X=xlone/Y=ylate/T=time ea105

DEFINE axis/modulo/x=0.25:359.75:0.5 xfine
DEFINE axis/y=-89.875:89.875:0.25 yfine
DEFINE grid/x=xfine/y=yfine grfine



file/skip=3/grid=na105/var="hdt,H,hbt,hfill" fort.201
file/skip=3/grid=ea105/var="hdt,H,hbt,hfill" fort.202

save/clob/file=NA.nc hdt[d=1], H[d=1], hbt[d=1], hfill[d=1]
save/clob/file=EA.nc hdt[d=2], H[d=2], hbt[d=2], hfill[d=2]


use topoGSMres.nc

!!  NA
! let hdt3crop = hdt[d=1, i=1:122]       !! Low res GSM
let hdt3crop = hdt[d=1, i=4:320]         !! High res GSM
let hdt3gl = hdt3crop[g=grfine]
let hdt3 = if hdt3gl gt -1000 then hdt3gl else (-1000)

!!  EA
! let hdt4crop = hdt[d=2, i=1:131, j=2:73]     !! Low res GSM
let hdt4crop = hdt[d=2, i=1:262, j=2:140]      !! High res GSM
let hdt4West = hdt4crop[x=13w:0]
let hdt4WestGl = hdt4West[g=grfine]
let hdt4East = hdt4crop[x=0:120e]
let hdt4EastGl = hdt4East[g=grfine]
let hdt4merge = if hdt4WestGl gt -1000 then hdt4WestGl else hdt4EastGl
let hdt4 = if hdt4merge gt -1000 then hdt4merge else (-1000)

!!  NA + EA
let hdtmerge = if hdt4 gt -1000 then hdt4 else hdt3

let tpc1 = if hdtmerge gt -1000 then hdtmerge else topoGSMres[d=3, gt=time]
let topoGSM = if tpc1 ge 0 then tpc1 else 0

!! Save GSM resolution topography to be passed to "regrid.sh"
save/clob/file=topoGSM.nc topoGSM


use ${IceLove}

let icefc3 = H[d=1, gxy=imask[d=4]@ave]
let ice3 = if icefc3 ge 0.3 then 1 else 0
let icefc4 = H[d=2, gxy=imask[d=4]@ave]
let ice4 = if icefc4 ge 0.3 then 1 else 0
! let myimask = if (imask[d=4, gt=time]+ice3+ice4) gt 0 then 1 else 0
let myimask = if (ice3+ice4) gt 0 then 1 else 0
save/clob/file=icemaskTemp.nc myimask[z=@ave]


!! Bering strait open/close check
let iceBlockArr = hdt[d=1,x=169.75w:168.75w,y=65.625:66.125@sum]
let iceblock = min( min(iceblockarr[x=169.75w],iceblockarr[x=169.25w]) , iceblockarr[x=168.75w] )

let bs = if iceBlock gt 0 then 0 else (-0.3)*hbt[d=1,x=169.75w:168.75w@ave,y=65.625:66.125@ave]/37.91
list/file="bs.dat"/clob/noheader bs
!! Bering strait check ends

EOF

${ferret} -script ffConvTopo.jnl

err=1

##################################################
### Modifying the "berg.nc" and "icemask.nc" to 
##  match Loveclim input format
log ncrename -O -v TOPOGSM,topoGSMres topoGSM.nc topoGSMres.nc
./regrid.sh ${upscl}

## Generate max/min topographies for lps calculation
# ./hMaxMin.sh
rm topoGSM.nc


## Generate slope files needed for next GSM run (fort.360 and fort.365)
cat <<EOF > ffSlope.jnl
define axis/x=0E:10w:10/modulo/unit=degree xGSM
define axis/y=87.5S:87.5N:5/unit=degree yGSM
def grid/x=xGSM/y=yGSM grGSM

file/var=slopex/col=36/grid=grGSM slopex10x5.dat
file/var=slopey/col=36/grid=grGSM slopey10x5.dat

!! NA+GR
set reg/x=185w:5e/y=30n:90
list/clob/file="fort.360"/noheader/order=xy/format=(2(f0.5,x)) slopex[d=1],slopey[d=2]

!! EA
set reg/x=25w:135/y=40n:90
list/clob/file="fort.365"/noheader/order=xy/format=(2(f0.5,x)) slopex[d=1],slopey[d=2]
EOF

${ferret} -script ffSlope.jnl

sed -i "1s/^/19 12 -180 0 32\.5 87\.5 10 5\n${nextOrby}\n\n/" fort.360
echo '111 ' >> fort.365
sed -i "1s/^/16 10 -20 130 42\.5 87\.5 10 5\n${nextOrby}\n\n/" fort.365
echo '111 ' >> fort.360



### Ice Mask
log ncrename -O -v MYIMASK,imask icemaskTemp.nc
## Find dimension names in icemaskTemp.nc
lon=`ncdump -h icemaskTemp.nc | grep -i lon | head -1 | awk '{print $1}'`
lat=`ncdump -h icemaskTemp.nc | grep -i lat | head -1 | awk '{print $1}'`
# time=`ncdump -h icemaskTemp.nc | grep -i time | head -1 | awk '{print $1}'`
# ncrename -O -d ${time},time -d ${lat},lat -d ${lon},lon -v ${time},time -v ${lat},lat -v ${lon},lon icemaskTemp.nc
log ncrename -O -d ${lat},lat -d ${lon},lon -v ${lat},lat -v ${lon},lon icemaskTemp.nc
log ncks -q -O -x -v lat,lon icemaskTemp.nc icemaskTemp2.nc
log ncks -q -A -v lat,lon ${IceLove} icemaskTemp2.nc
log ncap2 -O -s "time=${nextOrby}" icemaskTemp2.nc newIcemask.nc
log ncrename -O -d TIME,time newIcemask.nc

rm *Temp*

err=0

${execCpl}/trash ffConvTopo.jnl


## Runoff
cd ${execGSM}
${execCpl}/runoff.sh


cat<<EOF

### cnvGSMLoveEns.sh
##############################################################

EOF
