##############################################################
## NOTE:
## INDEPENDENT of fine topography
## Temperature, Precipitation, and Evaporation
##  are downscaled to fine grid using Ferret
## Temperature is the sea-level temperature (GSM requirement)
## Lapse-rate is the near fine surface lapse-rate
##
## export these variables before call:
##                                    ${orby}
##                                    ${execLove}
##                                    ${biasMethod}
##
## fort files passed to GSM:
##   NA: fort.311:315
##   EA: fort.321:325
##   SST NA: fort.310
##   SST EA: fort.320
##
##############################################################


##############################################################
## User Edit

# Results Storage Directory

atmOUTPath=${execLove}/OUTPUT/run/OUTPUT/run/STORAGE/LOVECLIM1.3/run/repo/output/atmos

## User Edit End
##############################################################


##############################################################
### ERROR HANDLING
##  err: Determines important errors from unimportant errors
##       Set it to a non-zero value before commands to stop
##        if they fail, and zero after them again
##       e.g.: err=1
##             command; err=0

err=0
error() {
    if [ ${err} -ne '0' ]; then
        echo "file $(basename $0) failed at line $1"
        exit 1
    fi
}
trap 'error ${LINENO}' ERR
##############################################################


log()
{
    eval $@
    if [[ ${debug} == 1 ]]; then
        echo $@
    fi
}


cat<<EOF
##############################################################
### cnvLoveGSMEns.sh

EOF

# Average over last "loveAve" years of each run
# (Can be modifiable later through the paramater file)
log ymax=${loveYearStep}
log ymin=$((loveYearStep-loveAve+1))

err=1

[[ -e ${atmOUTPath}/tmp ]] && rm -r ${atmOUTPath}/tmp
mkdir -p ${atmOUTPath}/tmp
cd ${atmOUTPath}/tmp
ln -s ../*.dat .
ln -s ${cplPDclimDir}/PDclim.nc .

## Converting from LoveCLim monthly mean values to
## GSM format

##########################################################
### Reading LoveClim output
##  Variable number in ${ferret} script
##  "coarseTopo.nc" is variable number 1
##  "fineTopo.nc" is variable number 2
##    3   4   5   6    7
## evap prec t2m t4g t tsDev
outVars=(evapMean precMean t2mMean t4gMean tsDev tsMean u800Mean u800Dev v800Mean v800Dev)

cat <<EOF > ffConvLoveGSM.jnl
! Define axis for reading LOVECLIM grid data
define axis/x=0E:5.625W:5.625/modulo/unit=degree xcoarse
define axis/y=87.1875S:87.1875N:5.625/unit=degree ycoarse
define axis/z=1:12:1 month
define axis/e=1:${ymax}:1 tmpYear
define axis/t=${nextOrby}:${nextOrby}:1 year
define grid/x=xcoarse/y=ycoarse/z=month/e=tmpYear/t=year grlove
EOF

## {var}mnthmn: each variable monthly mean for a full
##              100 years period
## Averaging over last n years of data for each variable
for var in ${outVars[*]}; do
    file=`ls ${var}*.dat | tail -n1`
    if [[ ${var} == 'tsMean' ]] || [[ ${var} == 't2mMean' ]] || [[ ${var} == 't4gMean' ]] || [[ ${var} == 't2gMean' ]] || [[ ${var} == 't0gMean' ]]; then
        extra="- 273.15"
    else
	extra=""
    fi
    cat <<EOF >> ffConvLoveGSM.jnl
file/var=${var}mnthmn/grid=grlove/order=xyzet ${file}
let ${var} = ${var}mnthmn[d=${file}, m=${ymin}:${ymax}@ave] ${extra}
EOF
done

##################################################
### Defining lapse-rate

if [[ ${lps} == '1' ]]; then
    cat <<EOF >> ffConvLoveGSM.jnl
let dpdz = 10.27
let/title="Lapse-rate using 1000kPa & 650hPa levels" \
lps = dpdz * (t2mMean[k=1:12] - t4gMean[k=1:12]) / (100000 - 65000) * 1000
EOF
elif [[ ${lps} == '0' ]]; then
    cat <<EOF >> ffConvLoveGSM.jnl
let/title="Fixed Lapse-rate" lps = 6.5 + 0 * t2mMean[k=1:12]
EOF
fi

####################################################################
## IF bias correction is activated, add more lines to the
## "ffConvLoveGSM.jnl" before running it.  Additional code lines are
## applied by "bias.sh".
cat <<EOF >> ffConvLoveGSM.jnl
! use "${cplCndDir}/bias/tsDiffs/ts_`echo ${orgNo} | sed 's/\(run[0-9]\{4\}\).*/\1/'`_months_1950-1980Diff.nc"
! let tCorrected  = t2mMean[k=1:12]  - tsbias[d=ts_`echo ${orgNo} | sed 's/\(run[0-9]\{4\}\).*/\1/'`_months_1950-1980Diff.nc, k=1:12]
let tCorrected  = t2mMean[k=1:12]
let precCorrected = precMean[k=1:12]
EOF



##################################################
### Defining each zone fine resolution region
##  Zones: North America (NA)
##         Eurasia       (EA)
##         Antarctica    (Ant)
##         Greenland     (Gr)

cat <<EOF >> ffConvLoveGSM.jnl

!! Resolution needed by GSM
!! NAGr:
DEFINE AXIS/X=180W:0:10/UNIT=DEGREE xlonNAGr
DEFINE AXIS/Y=32.5N:87.5N:5/UNIT=DEGREE ylatNAGr
DEFINE GRID/X=xlonNAGr/Y=ylatNAGr/T=year  grNAGr

!! EA:
DEFINE AXIS/modulo/X=20W:130E:10/UNIT=DEGREE xlonEA
DEFINE AXIS/Y=42.5N:87.5N:5/UNIT=DEGREE ylatEA
DEFINE GRID/X=xlonEA/Y=ylatEA/T=year  grEA


! !! High res
! !! NA:
! DEFINE AXIS/X=173W:3W:2/UNIT=DEGREE xlonNGf5
! DEFINE AXIS/Y=34N:86N:1/UNIT=DEGREE ylatNGf25
! DEFINE GRID/X=xlonNGf5/Y=ylatNGf25/T=year  grfineNA

! !! EA:
! DEFINE AXIS/modulo/X=14W:122E:2/UNIT=DEGREE xlone5f
! DEFINE AXIS/Y=47N:84N:1/UNIT=DEGREE ylate25f
! DEFINE GRID/X=xlone5f/Y=ylate25f/T=year  grfineEA


! !! Low res
! !! NA:
! ! DEFINE AXIS/X=174W:40W:2/UNIT=DEGREE xlonNGf5
! ! DEFINE AXIS/Y=34N:86N:1/UNIT=DEGREE ylatNGf25
! ! DEFINE GRID/X=xlonNGf5/Y=ylatNGf25/T=year  grfineNA

! !! EA:
! ! DEFINE AXIS/modulo/X=14W:122E:2/UNIT=DEGREE xlone5f
! ! DEFINE AXIS/Y=46N:84N:1/UNIT=DEGREE ylate25f
! ! DEFINE GRID/X=xlone5f/Y=ylate25f/T=year  grfineEA

EOF

# ##################################################
# ### Downscaling
# ##  I guess in Loveclim snowfall is in the same
# ##  unit as rainfall (they can be added)!

regList=(NAGr EA)
GSMvarList=(tCorrected lps precCorrected evapMean tsDev u800Mean u800Dev v800Mean v800Dev)
outVars+=('lps')

### Global Coarse Resolution #######################
# Save in netCDF format
printf 'save/clob/file="output.nc" ' >> ffConvLoveGSM.jnl
for var in ${outVars[*]}; do
    printf '%s, ' "${var}" >> ffConvLoveGSM.jnl
done
sed -i '$ s/,\ $//' ffConvLoveGSM.jnl
printf '\n' >> ffConvLoveGSM.jnl


for reg in ${regList[*]}; do
### Regional Fine Resolution #######################
    # Save variables needed by GSM in text format
    for var in ${GSMvarList[*]}; do
	cat <<EOF >> ffConvLoveGSM.jnl
list/noheader/clob/order=zxy/file="${var}${reg}.dat"/format=(11(f6.1,1x)f6.1) ${var}[gxy=gr${reg}@ave]
EOF
    done


### Regional Fine Resolution #######################
    # Save in netCDF format
    printf 'save/clob/file="output%s.nc" ' "${reg}" >> ffConvLoveGSM.jnl
    for var in ${GSMvarList[*]}; do
	printf '%s, ' "${var}[gxy=gr${reg}@ave]" >> ffConvLoveGSM.jnl
    done
    sed -i '$ s/,\ $//' ffConvLoveGSM.jnl
    printf '\n' >> ffConvLoveGSM.jnl
done

##################################################
### Running Ferret scripts & Remove them
${ferret} -memsize 100 -script ffConvLoveGSM.jnl
${execCpl}/trash ffConvLoveGSM.jnl
rm `ls *.dat | grep -v NA | grep -v EA`

##################################################
### Rename new files to fort.21*
### Change the header of new files to match 
###  GSM needs

## NA region attributes (2nd line)
## files: fort.211,3,5,7
## High res
# l2NA="86 53 -173.0 -3.0 34 86 2.0 1.0"
l2NAGr="19 12 -180.0 0.0 32.5 87.5 10.0 5.0"
## Low res
# l2NA="67 52 -174.0 -40.0 34.0 86.0 2.0 1.0"

## EA region attributes (2nd line)
## files: fort.212,4,6,8
## High res
# l2EA="69 38 -14.0 122.0 47.0 84.0 2.0 1.0"
l2EA="16 10 -20.0 130.0 42.5 87.5 10.0 5.0"
## Low res
# l2EA="68 38 -14.0 122.0 46.0 84.0 2.0 1.0"


log mv tCorrectedNAGr.dat fort.311
sed -i 1i"\# vTgcm, t2mNA.dat, NA\n${l2NAGr}\n${orby}\n" fort.311

log mv lpsNAGr.dat fort.312
sed -i 1i"\# vklapsgcm, lpsNA.dat, NA\n${l2NAGr}\n${orby}\n" fort.312

log mv precCorrectedNAGr.dat fort.313
sed -i 1i"\# vPgcm, precNA.dat, NA\n${l2NAGr}\n${orby}\n" fort.313

log mv evapMeanNAGr.dat fort.314
sed -i 1i"\# vevapgcm, evapNA.dat, NA\n${l2NAGr}\n${orby}\n" fort.314

log mv tsDevNAGr.dat fort.315
sed -i 1i"\# vTsigmagcm, tsigmaNA.dat, NA\n${l2NAGr}\n${orby}\n" fort.315

log mv tCorrectedEA.dat fort.321
sed -i 1i"\# vTgcm, t2mEA.dat, EA\n${l2EA}\n${orby}\n" fort.321

log mv lpsEA.dat fort.322
sed -i 1i"\# vklapsgcm, lpsEA.dat, EA\n${l2EA}\n${orby}\n" fort.322

log mv precCorrectedEA.dat fort.323
sed -i 1i"\# vPgcm, precEA.dat, EA\n${l2EA}\n${orby}\n" fort.323

log mv evapMeanEA.dat fort.324
sed -i 1i"\# vevapgcm, evapEA.dat, EA\n${l2EA}\n${orby}\n" fort.324

log mv tsDevEA.dat fort.325
sed -i 1i"\# vTsigmagcm, tsigmaEA.dat, EA\n${l2EA}\n${orby}\n" fort.325


## Wind
log mv u800MeanNAGr.dat fort.361
sed -i "1s/^/${l2NAGr}\n${orby}\n\n/" fort.361
echo '111 ' >> fort.361

log mv u800DevNAGr.dat fort.362
sed -i "1s/^/${l2NAGr}\n${orby}\n\n/" fort.362
echo '111 ' >> fort.362

log mv v800MeanNAGr.dat fort.363
sed -i "1s/^/${l2NAGr}\n${orby}\n\n/" fort.363
echo '111 ' >> fort.363

log mv v800DevNAGr.dat fort.364
sed -i "1s/^/${l2NAGr}\n${orby}\n\n/" fort.364
echo '111 ' >> fort.364


log mv u800MeanEA.dat fort.366
sed -i "1s/^/${l2EA}\n${orby}\n\n/" fort.366
echo '111 ' >> fort.366

log mv u800DevEA.dat fort.367
sed -i "1s/^/${l2EA}\n${orby}\n\n/" fort.367
echo '111 ' >> fort.367

log mv v800MeanEA.dat fort.368
sed -i "1s/^/${l2EA}\n${orby}\n\n/" fort.368
echo '111 ' >> fort.368

log mv v800DevEA.dat fort.369
sed -i "1s/^/${l2EA}\n${orby}\n\n/" fort.369
echo '111 ' >> fort.369


## Archive LOVECLIM *.dat files
log cd ${atmOUTPath}
log tar -czf loveOut.`printf %06d ${loveYear}`.tar.gz *.dat
rm *.dat



##################################################
### Extract the temperature between -50 and -600 m 
### at certain points

ocnOUTPath=${execLove}/OUTPUT/run/OUTPUT/run/STORAGE/LOVECLIM1.3/run/repo/output/ocean
log cd ${ocnOUTPath}

## Create spec file
cat <<EOF > ToceanSPECv1.NA.dat
## ToceanSPECv1.NA.dat kref,x1,x2,y1,y2,xref,yref

6 752
1 -135.0 -20.0 72.0 85.0 -135.0 72.0
2 -45.0 -15.0 59.0 85.0 -15.0 76.0
3 -78.0 -45.0 59.0 76.0 -48.0 59.0
4 -75.0 -58.0 60.0 75.0 -75.0 75.0
5 -60.0 -45.0 45.0 60.0 -60.0 60.0
6 -65.0 -50.0 40.0 48.0 -65.0 40.0
EOF

cat <<EOF > ToceanSPECv1.EA.dat
## ToceanSPECv1.EA.dat kref,x1,x2,y1,y2,xref,yref

4 753
1 -14.0 5.0 52.0 62.0 -9.0 52.0
2 2.0 17.0 63.0 73.0 2.0 63.0
3 17.0 40.0 65.0 73.0 17.0 73.0
4 10.0 100.0 76.0 82.0 58.0 81.0
EOF


## Create fort files

log ${execCpl}/ToceanExt.py `ls CLIO3m.* | tail -1` ${orby} ${loveYearStep} ${loveAve}
sed '1s/# //' out.na | sed '2s/^/\n/' > fort.310
sed '1s/# //' out.ea | sed '2s/^/\n/' > fort.320


cat<<EOF

### cnvLoveGSMEns.sh
##############################################################

EOF

