// NetCDFwavg -- NetCDF weighted average 
// is based on ncfint 1.29 developped under GPL licence by
//    Charlie Zender
//    Department of Earth System Science
//    University of California at Irvine
//    Irvine, CA 92697-3100

#include <math.h>
#include <string>
#include <iostream>
#include <vector>

#include <stdlib.h>

#include <netcdf.h>

#define VERBOSE 0

using namespace std;

class variable
{
  public:
    string name;
    int id;
    int nbdim;
    long size;
    nc_type type;
    vector<int> iddim;
    variable(string namein, int idin) { id=idin; name=namein; nbdim=0; size=1; };
    void adddim(int idin, long sizein) { iddim.push_back(idin); nbdim++; size*= sizein; };
};

class fichier
{
  public:
    string name;
    double weight;
    int id;
    vector<variable> var;
    vector<variable> dim;
    vector<bool> isAlsoDim;
    int nbvar;
    int nbdim;
    int idRecDim;
    
    fichier(string namein) { name=namein; weight=1.; nbvar=0; nbdim=0; idRecDim=-1; };
    fichier(string namein, double weightin) { name=namein; weight=weightin; nbvar=0; nbdim=0; idRecDim=-1; };
    void addvar(string varnamein, int idin, nc_type typein) { var.push_back(variable(varnamein, idin)); var[var.size()-1].type=typein; nbvar++; };
    void adddim(string dimnamein, int idin, long sizein) { dim.push_back(variable(dimnamein, idin)); dim[dim.size()-1].size=sizein; nbdim++; };
    void checkDimInVar()
    {
      for(int i=0; i<(int)var.size(); i++)
      {
        int j=0;
        while(j<nbdim && dim[j].name!=var[i].name)
          j++;
        if(j==nbdim) 
          isAlsoDim.push_back(false);
        else
          isAlsoDim.push_back(true);
      }
    }
    void print()
    {
      if(VERBOSE!=0)
      {
	cout<<name<<" (weight normalized="<<weight<<") :"<<endl;
	if(VERBOSE==1)
	{
	  cout<<"  "<<nbdim<<" dimension(s) and "<<nbvar<<" variable(s)"<<endl;
	}else{
	  cout<<"  Dimension :"<<endl;
	  for(int i=0; i<nbdim; i++) cout<<"    "<<dim[i].id<<") "<<dim[i].name<<" - "<<dim[i].size<<" element(s)"<<endl;
	  
	  cout<<"  Variable :"<<endl;
	  for(int i=0; i<nbvar; i++)
	  {
	    cout<<"    "<<var[i].id<<") "<<var[i].name<<"(";
	    for(int j=0; j<var[i].nbdim; j++)
	    {
	      if(j<var[i].nbdim-1)
	      {
		cout<<dim[var[i].iddim[j]].name<<", ";
	      }else{
		cout<<dim[var[i].iddim[j]].name;
	      }
	    }
	    cout<<")"<<endl;
	  }
	}
      }
    };
};

double randg(double sigma)
{
  double x,y,r2;
  static double m=2./RAND_MAX;
 
  if (sigma==0) return(rand()*m-1.);
  do {
        x=rand()*m-1.;
    y=rand()*m-1.;
        r2=x*x+y*y;
  } while (r2>1.0 || r2==0);
  return (sigma*y*sqrt(-2.*log(r2)/r2));
}

void PutNoise(double *out, void *in, double Sigma, long size, nc_type type)
{
  short *inS;
  int *inI;
  float *inF;
  double *inD;
  switch(type)
  {
    case(3):
      inS=(short*)in;
      for(long i=0; i<size; i++)
      {
        double random=randg(Sigma);
        out[i]=(double)(inS[i])+random;
      }
      break;
    case(4):
      inI=(int*)in;
      for(long i=0; i<size; i++)
      {
        double random=randg(Sigma);
        out[i]=(double)(inI[i])+random;
      }
      break;
    case(5):
      inF=(float*)in;
      for(long i=0; i<size; i++)
      {
        double random=randg(Sigma);
        out[i]=(double)(inF[i])+random;
      }
      break;
    case(6):
      inD=(double*)in;
       for(long i=0; i<size; i++)
      {
        double random=randg(Sigma);
        out[i]=(double)(inD[i])+random;
      }
      break;
    default:
      cout<<"Unknown type !!!"<<endl;
  }
}

void ConvertfromDouble(void *out, double *in, long size, nc_type type)
{
  short *outS;
  int *outI;
  float *outF;
  double *outD;
  switch(type)
  {
    case(3):
      outS=(short*)out;
      for(long i=0; i<size; i++)outS[i]=(short)(roundl(in[i]));
      break;
    case(4):
      outI=(int*)out;
      for(long i=0; i<size; i++)outI[i]=(int)(roundl(in[i]));
      break;
    case(5):
      outF=(float*)out;
      for(long i=0; i<size; i++)outF[i]=in[i];
      break;
    case(6):
      outD=(double*)out;
      for(long i=0; i<size; i++)outD[i]=in[i];
      break;
    default:
      cout<<"Unknown type !!!"<<endl;
  }
}

int main(int argc,char **argv)
{
  if(argc!=4)
  {
    cout<<"Usage :"<<endl<<"\t./BruitXXX input.nc output.nc sigma"<<endl;
    exit(0);
  }
  
  srand(10);

  
  
  fichier file(argv[1]);
  double sigma=atof(argv[3]);
  
    //open the file
    file.id=ncopen(file.name.c_str(),NC_NOWRITE);
    
    //Get the number of variables and dimensions in the file
    int nbdim;
    int nbvar;
    ncinquire(file.id, &nbdim, &nbvar, NULL, &file.idRecDim);
    

    //Form initial extraction list from user input
    for(int j=0; j<nbvar; j++)
    {
      //Get the name for each variable.
      char varname[MAX_NC_NAME];
      nc_type type;
      ncvarinq(file.id, j, varname, &type, NULL, NULL, NULL);
      file.addvar(varname, j, type);
    }
     
    //Add all coordinate variables to extraction list
    for(int j=0; j<nbdim; j++)
    {
      char dimname[MAX_NC_NAME];
      long size;
      ncdiminq(file.id, j, dimname, &size);
      file.adddim(dimname, j, size);
    }
    
    //rempli tableau isAlsoDim
    file.checkDimInVar();

    //Get the number of dimensions
    for(int j=0; j<file.nbvar; j++)
    {
      int nbvardim;
      int dimidtab[MAX_NC_DIMS];
      ncvarinq(file.id, file.var[j].id, NULL, NULL, &nbvardim, dimidtab, NULL);
      for(int k=0; k<nbvardim; k++) file.var[j].adddim(dimidtab[k], file.dim[dimidtab[k]].size);
    }
    file.print();
  
     //Open output file
  int outid=nccreate(argv[2], NC_CLOBBER);
    
  //Copy global attributes
  int nbatt;
  ncinquire(file.id, NULL, NULL, &nbatt, NULL);  
  for(int i=0; i<nbatt; i++)
  {
    char attname[MAX_NC_NAME];
    ncattname(file.id, NC_GLOBAL, i, attname);
    ncattcopy(file.id, NC_GLOBAL, attname, outid, NC_GLOBAL);
  }
    
  //Define dimensions in output file
  for(int i=0; i<file.nbdim; i++)
  {
    if(file.idRecDim==file.dim[i].id)
    {
      ncdimdef(outid, file.dim[i].name.c_str(), NC_UNLIMITED);
    }else{
      ncdimdef(outid, file.dim[i].name.c_str(), file.dim[i].size);
    }
  }
  
  //Define variables in output file, and copy their attributes
  for(int i=0; i<file.nbvar; i++)
  {
    int dimtab[MAX_NC_DIMS];
    for(int j=0; j<file.var[i].nbdim; j++) dimtab[j]=file.var[i].iddim[j];
    int outvarid=ncvardef(outid, file.var[i].name.c_str(), file.var[i].type, file.var[i].nbdim, dimtab);

    int nbatt;
    ncvarinq(file.id, i, NULL, NULL, NULL, NULL, &nbatt);
    for(int j=0; j<nbatt; j++)
    {
      char attname[MAX_NC_NAME];
      ncattname(file.id, file.var[i].id, j, attname);
      ncattcopy(file.id, file.var[i].id, attname, outid, outvarid);
    }
  }
  
  //Turn off default filling behavior to enhance efficiency
  ncsetfill(outid, NC_NOFILL);
  
  //Take output file out of define mode
  ncendef(outid);

  //calc
  for(int j=0; j<file.nbvar; j++)
  {
    long start[file.var[j].nbdim];
    long count[file.var[j].nbdim];
    long totalsize=1;
    for(int k=0; k<file.var[j].nbdim; k++)
    {
      start[k]=0;
      count[k]=file.dim[file.var[j].iddim[k]].size;
      totalsize*=count[k];
    }
    void *valuein = (void *)malloc(totalsize*nctypelen(file.var[j].type));
    double *valuecalc = (double *)malloc(totalsize*sizeof(double));
    void *valueout = (void *)malloc(totalsize*nctypelen(file.var[j].type));
    
    ncvarget(file.id, file.var[j].id, start, count, valuein);
    if(file.isAlsoDim[j])
    {
      ncvarput(outid, file.var[j].id, start, count, valuein);
    }else{
      PutNoise(valuecalc, valuein, sigma, totalsize, file.var[j].type);
      ConvertfromDouble(valueout, valuecalc, totalsize, file.var[j].type);
      ncvarput(outid, file.var[j].id, start, count, valueout);
    }    
  }
  
  cout<<"==> Avg file "<<argv[2]<<" created with sigma="<<sigma<<endl;
  
  //Close input netCDF files
  ncclose(file.id);
  ncclose(outid);  

  return 0;
}
