// NetCDFwavg -- NetCDF weighted average 
// is based on ncfint 1.29 developped under GPL licence by
//    Charlie Zender
//    Department of Earth System Science
//    University of California at Irvine
//    Irvine, CA 92697-3100

#include <math.h>
#include <string>
#include <iostream>
#include <vector>
#include <iomanip>
#include <stdlib.h>
#include <netcdf.h>

#define VERBOSE 1

using namespace std;

class variable
{
  public:
    string name;
    int id;
    int nbdim;
    long size;
    nc_type type;
    vector<int> iddim;
    variable(string namein, int idin) { id=idin; name=namein; nbdim=0; size=1; };
    void adddim(int idin, long sizein) { iddim.push_back(idin); nbdim++; size*= sizein; };
};

class fichier
{
  public:
    string name;
    double weight;
    int id;
    vector<variable> var;
    vector<variable> dim;
    int nbvar;
    int nbdim;
    int idRecDim;
    
    fichier(string namein) { name=namein; weight=1.; nbvar=0; nbdim=0; idRecDim=-1; };
    fichier(string namein, double weightin) { name=namein; weight=weightin; nbvar=0; nbdim=0; idRecDim=-1; };
    void addvar(string varnamein, int idin, nc_type typein) { var.push_back(variable(varnamein, idin)); var[var.size()-1].type=typein; nbvar++; };
    void adddim(string dimnamein, int idin, long sizein) { dim.push_back(variable(dimnamein, idin)); dim[dim.size()-1].size=sizein; nbdim++; };
    void print()
    {
      if(VERBOSE!=0)
      {
	cout<<name<<" :"<<endl;
	if(VERBOSE==1)
	{
	  cout<<"  "<<nbdim<<" dimension(s) and "<<nbvar<<" variable(s)"<<endl;
	}else{
	  cout<<"  Dimension :"<<endl;
	  for(int i=0; i<nbdim; i++) cout<<"    "<<dim[i].id<<") "<<dim[i].name<<" - "<<dim[i].size<<" element(s)"<<endl;
	  
	  cout<<"  Variable :"<<endl;
	  for(int i=0; i<nbvar; i++)
	  {
	    cout<<"    "<<var[i].id<<") "<<var[i].name<<"(";
	    for(int j=0; j<var[i].nbdim; j++)
	    {
	      if(j<var[i].nbdim-1)
	      {
		cout<<dim[var[i].iddim[j]].name<<", ";
	      }else{
		cout<<dim[var[i].iddim[j]].name;
	      }
	    }
	    cout<<")"<<endl;
	  }
	}
      }
    };
};


int main(int argc,char **argv)
{
  
  static const double latECBilt[32]={
    -85.76059, -80.26878, -74.74454, -69.21297, -63.67863, -58.14296,
    -52.60653, -47.06964, -41.53246, -35.99508, -30.45755, -24.91993, 
    -19.38223, -13.84448, -8.306703, -2.768903, 2.768903, 8.306703, 13.84448, 
    19.38223, 24.91993, 30.45755, 35.99508, 41.53246, 47.06964, 52.60653, 
    58.14296, 63.67863, 69.21297, 74.74454, 80.26878, 85.76059};
  static const double lonECBilt[64]={
    0, 5.625, 11.25, 16.875, 22.5, 28.125, 33.75, 39.375, 45, 50.625, 
    56.25, 61.875, 67.5, 73.125, 78.75, 84.375, 90, 95.625, 101.25, 106.875, 
    112.5, 118.125, 123.75, 129.375, 135, 140.625, 146.25, 151.875, 157.5, 
    163.125, 168.75, 174.375, 180, 185.625, 191.25, 196.875, 202.5, 208.125, 
    213.75, 219.375, 225, 230.625, 236.25, 241.875, 247.5, 253.125, 258.75, 
    264.375, 270, 275.625, 281.25, 286.875, 292.5, 298.125, 303.75, 309.375, 
    315, 320.625, 326.25, 331.875, 337.5, 343.125, 348.75, 354.375};

  if(argc!=3)
  {
    cout<<"Usage:"<<endl;
    cout<<"\tChangeGrid [input file] [output file]"<<endl;
    return 1;
  }
  
  int nlon=64; 
  int nlat=32;
  
  
  fichier file(fichier(argv[1],1));
   
  //open the file
  file.id=ncopen(file.name.c_str(),NC_NOWRITE);
  
  //Get the number of variables and dimensions in the file
  int nbdim;
  int nbvar;
  ncinquire(file.id, &nbdim, &nbvar, NULL, &file.idRecDim);
    

  //Form initial extraction list from user input
  for(int j=0; j<nbvar; j++)
  {
    //Get the name for each variable.
    char varname[MAX_NC_NAME];
    nc_type type;
    ncvarinq(file.id, j, varname, &type, NULL, NULL, NULL);
    file.addvar(varname, j, type);
  }
     
  //Add all coordinate variables to extraction list
  for(int j=0; j<nbdim; j++)
  {
    char dimname[MAX_NC_NAME];
    long size;
    ncdiminq(file.id, j, dimname, &size);
    file.adddim(dimname, j, size);
  }
      
  //Get the number of dimensions
  for(int j=0; j<file.nbvar; j++)
  {
    int nbvardim;
    int dimidtab[MAX_NC_DIMS];
    ncvarinq(file.id, file.var[j].id, NULL, NULL, &nbvardim, dimidtab, NULL);
    for(int k=0; k<nbvardim; k++) file.var[j].adddim(dimidtab[k], file.dim[dimidtab[k]].size);
  }
  file.print();
  
  //Open output file
  int outid=nccreate(argv[2], NC_CLOBBER);
    
  //Copy global attributes
  int nbatt;
  ncinquire(file.id, NULL, NULL, &nbatt, NULL);  
  for(int i=0; i<nbatt; i++)
  {
    char attname[MAX_NC_NAME];
    ncattname(file.id, NC_GLOBAL, i, attname);
    ncattcopy(file.id, NC_GLOBAL, attname, outid, NC_GLOBAL);
  }
  
  //check format HadCRUT3
  cout<<"Check input file format...";
  if(file.dim[0].name!="longitude"){cout<<"Not a HadCRUT3 format (dim 0 != longitude)"<<endl; return 1;}
  if(file.dim[1].name!="latitude"){cout<<"Not a HadCRUT3 format (dim 1 != latitude)"<<endl; return 1;}
  if(file.dim[2].name!="unspecified"){cout<<"Not a HadCRUT3 format (dim 2 != unspecified)"<<endl; return 1;}
  if(file.dim[3].name!="t"){cout<<"Not a HadCRUT3 format (dim 3 != t)"<<endl; return 1;}
  if(file.var[0].name!="longitude"){cout<<"Not a HadCRUT3 format (var 0 != longitude)"<<endl; return 1;}
  if(file.var[1].name!="latitude"){cout<<"Not a HadCRUT3 format (var 1 != latitude)"<<endl; return 1;}
  if(file.var[2].name!="unspecified"){cout<<"Not a HadCRUT3 format (var 2 != unspecified)"<<endl; return 1;}
  if(file.var[3].name!="t"){cout<<"Not a HadCRUT3 format (var 3 != t)"<<endl; return 1;}
  if(file.var[4].name!="temp"){cout<<"Not a HadCRUT3 format (var 4 != temp)"<<endl; return 1;}
  if(file.var[4].nbdim!=4){cout<<"Not a HadCRUT3 format (var 4 have not 4 dim)"<<endl; return 1;}
  cout<<"[OK]"<<endl;
  
  
  //Define dimensions in output file
  ncdimdef(outid, "lon", nlon);
  ncdimdef(outid, "lat", nlat);
  ncdimdef(outid, "time", NC_UNLIMITED);
    
  //Define variables in output file, and copy their attributes
  int dimtab[MAX_NC_DIMS];
  int outvarid;
  
  dimtab[0]=0;
  outvarid=ncvardef(outid, "lon", NC_FLOAT, 1, dimtab);
  ncattput(outid, outvarid, "long_name", NC_CHAR, 9, "longitude");
  ncattput(outid, outvarid, "standard_name", NC_CHAR, 3, "lon");
  ncattput(outid, outvarid, "units", NC_CHAR, 12, "degrees_east");
  ncattput(outid, outvarid, "axis", NC_CHAR, 1, "X");
  
  dimtab[0]=1;
  outvarid=ncvardef(outid, "lat", NC_FLOAT, 1, dimtab);
  ncattput(outid, outvarid, "long_name", NC_CHAR, 8, "latitude");
  ncattput(outid, outvarid, "standard_name", NC_CHAR, 3, "lat");
  ncattput(outid, outvarid, "units", NC_CHAR, 13, "degrees_north");
  ncattput(outid, outvarid, "axis", NC_CHAR, 1, "Y");
  
  dimtab[0]=2;
  outvarid=ncvardef(outid, "time", NC_FLOAT, 1, dimtab);
  ncattput(outid, outvarid, "long_name", NC_CHAR, 4, "time");
  ncattput(outid, outvarid, "standard_name", NC_CHAR, 4, "time");
  ncattput(outid, outvarid, "units", NC_CHAR, 30, "days since 1850-01-01 00:00:00");
  ncattput(outid, outvarid, "time_origin", NC_CHAR, 20, "01-JAN-1850:00:00:00");
  
  dimtab[0]=2; dimtab[1]=1; dimtab[2]=0;
  outvarid=ncvardef(outid, "ts", NC_FLOAT, 3, dimtab);
  ncattput(outid, outvarid, "long_name", NC_CHAR, 10, "ts_anomaly");
  ncattput(outid, outvarid, "standard_name", NC_CHAR, 2, "ts");
  ncattput(outid, outvarid, "units", NC_CHAR, 1, "K");
  float *value=new float[1]; value[0]=-1E+32; ncattput(outid, outvarid, "missing_value", NC_FLOAT, 1, value);
  free(value);  
 
  //Turn off default filling behavior to enhance efficiency
  ncsetfill(outid, NC_NOFILL);
  
  //Take output file out of define mode
  ncendef(outid);

  //output value
  long start[MAX_NC_DIMS];
  long count[MAX_NC_DIMS];
  long countout[MAX_NC_DIMS];
  
  //lon
  start[0]=0;
  countout[0]=nlon;
  value = (float *)malloc(countout[0]*nctypelen(NC_FLOAT));
  for(int ii=0; ii<nlon; ii++)
      value[ii]=lonECBilt[ii];
  ncvarput(outid, 0, start, countout, value);
  free(value);
  
  //lat
  start[0]=0;
  countout[0]=nlat;
  value = (float *)malloc(countout[0]*nctypelen(NC_FLOAT));
  for(int ii=0; ii<nlat; ii++)
      value[ii]=latECBilt[ii];
  ncvarput(outid, 1, start, countout, value);
  free(value);
  
  //time
  start[0]=0;
  count[0]=file.dim[3].size;
  countout[0]=file.dim[3].size;
  value = (float *)malloc(count[0]*nctypelen(NC_FLOAT));
  ncvarget(file.id, file.var[3].id, start, count, value);
  ncvarput(outid, 2, start, countout, value);
  free(value);
  
  //ts
  start[0]=0;
  long intotalsize=file.dim[0].size*file.dim[1].size*file.dim[3].size;
  count[3]=file.dim[0].size; count[2]=file.dim[1].size; count[1]=file.dim[2].size; count[0]=file.dim[3].size;
  countout[2]=nlon; countout[1]=nlat; count[0]=file.dim[3].size;
  value = (float *)malloc(intotalsize*nctypelen(NC_FLOAT));
  ncvarget(file.id, file.var[4].id, start, count, value);

  double ****intab;
  intab = new double***[count[0]];
  for(int ii=0; ii<count[0]; ii++)
  {
    intab[ii] = new double**[count[1]];
    for(int jj=0; jj<count[1]; jj++)
    {
      intab[ii][jj] = new double*[count[2]];
      for(int kk=0; kk<count[2]; kk++)
        intab[ii][jj][kk] = new double[count[3]];
    }
  }     
        
  double ***outtab;
  outtab = new double**[countout[0]];
  for(int t=0; t<countout[0]; t++)
  {
    outtab[t] = new double*[countout[1]];
    for(int ilat=0; ilat<countout[1]; ilat++)
      outtab[t][ilat] = new double[countout[2]];
  } 
        
  for(int t=0; t<count[0]; t++)
    for(int ilat=0; ilat<count[2]; ilat++)
      for(int ilon=0; ilon<count[3]; ilon++)
      {
        long idx=ilon+ilat*count[3]+t*count[3]*count[2];
        if(value[idx]>2.00000004008175E20 && value[idx]<2.00000004008176E20) //merci le fortran de m....
          intab[t][0][ilat][ilon]=-1E+32;
        else
          intab[t][0][ilat][ilon]=value[idx];
      }
      
        
  //Interpo sur la grille ECBilt
  double spv=-1E+32;
  for(int t=0; t<count[0]; t++)
  {
    for(int mlat=0; mlat<nlat; mlat++)
      for(int mlon=0; mlon<nlon; mlon++)
        outtab[t][mlat][mlon]=0;
    
    cout.flush();
    cout<<"\r";
    for(int pourcent=0; pourcent<30; pourcent++)
      if(pourcent<30*(t/(double)count[0]))
        cout<<"#";
      else
        cout<<"_";
    cout.flush();
    
    for(int mlat=0; mlat<nlat; mlat++)
      for(int mlon=0; mlon<nlon; mlon++)
      {
        int compt=0;
        double x1=-2.81+(mlon*5.625);
        double x2=x1+5.625;
        double y1=-90+(mlat*5.625);
        double y2=y1+5.625;

        for(int ilat=0; ilat<36; ilat++)
          for(int ilon=0; ilon<72; ilon++)
            if(intab[t][0][ilat][ilon]!=spv)
            {
              double xi=-177.5+(ilon*5.);
              if(xi<0) xi+=360;
              double yi=-87.5+(ilat*5.);
              if( ((xi<x2) && (xi>x1)) && ((yi<y2) && (yi>y1)))
              {
                outtab[t][mlat][mlon]+=intab[t][0][ilat][ilon];
                compt++;
              }
            }  
        
        if(compt==0)
          outtab[t][mlat][mlon]=spv;
        else
          outtab[t][mlat][mlon]/=(double)compt;
      } 
  }

  for(int t=0; t<countout[0]; t++)
    for(int ilat=0; ilat<countout[1]; ilat++)
      for(int ilon=0; ilon<countout[2]; ilon++)
      {
        long idx=ilon+ilat*countout[2]+t*countout[1]*countout[2];
        value[idx]=outtab[t][ilat][ilon];
      }
      
  ncvarput(outid, 3, start, countout, value);

  
  if(VERBOSE>=1)
  {
    cout<<"==> File "<<argv[2]<<" created"<<endl;
  }
  
  //Close input netCDF files
  ncclose(file.id);
  ncclose(outid);  

  return 0;
}
