#include <math.h>
#include <string>
#include <iostream>
#include <vector>
#include <fstream>
#include <stdlib.h>
#include <netcdf.h>

#define VERBOSE 2

using namespace std;

void check(int errorcode, const int line, const char *file)
{
  if (errorcode != NC_NOERR)
  {
    cerr<<"line "<<line<<" of "<<file<<" : "<<nc_strerror(errorcode)<<endl;
    exit(1);
  }
};

void AccumulateMean(double *out, void *in, double w, long size, nc_type type);
void addCte(void *table, void *cte, long size, nc_type type);
void ConvertfromDouble(void *out, double *in, long size, nc_type type);
bool is_readable(const string &file);

class variable
{
  public:
    string name;
    int id;
    int nbdim;
    long size;
    nc_type type;
    vector<int> iddim;
    variable(string namein, int idin) { id=idin; name=namein; nbdim=0; size=1; };
    void adddim(int idin, long sizein) { iddim.push_back(idin); nbdim++; size*= sizein; };
};

class fichier
{
  public:
    string name;
    double weight;
    int id;
    vector<variable> var;
    vector<variable> dim;
    int nbvar;
    int nbdim;
    int idRecDim;
    int idRecVar;
    int isCreate;

    fichier(string namein) { name=namein; weight=1; nbvar=0; nbdim=0; idRecDim=-1; idRecVar=-1; };
    fichier(string namein, int weightin) { name=namein; weight=weightin; nbvar=0; nbdim=0; idRecDim=-1; idRecVar=-1; };
    void open(int Mode) { isCreate=0; check(nc_open(name.c_str(), Mode, &id),__LINE__,__FILE__); };
    void create(int Mode) { isCreate=1; check(nc_create(name.c_str(), Mode, &id),__LINE__,__FILE__); };
    void WriteSturcture() { /*check(nc_set_fill(id, NC_NOFILL,NULL),__LINE__,__FILE__); if(isCreate) check(nc_enddef(id),__LINE__,__FILE__)*/; };
    void addvar(string varnamein, nc_type typein, int dimtab[MAX_NC_DIMS], const int nbVarDim)
    {
      size_t chunks[nbVarDim];
      var.push_back(variable(varnamein, nbvar));
      var[var.size()-1].type=typein; nbvar++;
      for(int i=0; i<nbVarDim; i++)
      {
        if(isCreate)
          var[nbvar-1].adddim(dimtab[i], 0);
        else
          var[nbvar-1].adddim(dimtab[i], dim[dimtab[i]].size);
        chunks[i]=dim[dimtab[i]].size;
      }
      if(isCreate) 
      {
        int varid;
        check(nc_def_var(id, varnamein.c_str(), typein, nbVarDim, dimtab, &varid),__LINE__,__FILE__);
        //if(dimtab[0]!=idRecDim)
        //{
          //cout<<varnamein<<"="<<nc_def_var_chunking(id, varid, NC_CHUNKED, chunks)<<"<>"<<NC_EINVAL<<endl;
          check(nc_def_var_deflate(id, varid, NC_SHUFFLE, /*deflate*/0, /*deflate_level*/1),__LINE__,__FILE__);
        //}
      }
    };
    void adddim(string dimnamein, long sizein, int isIdRecDim)
    {
      dim.push_back(variable(dimnamein, nbdim));
      dim[dim.size()-1].size=sizein; nbdim++; 
      
      if(isCreate)
      {
        if(isIdRecDim)
        {
          check(nc_def_dim(id, dimnamein.c_str(), NC_UNLIMITED, NULL),__LINE__,__FILE__);
          idRecDim=dim.size()-1;
        }else{
          check(nc_def_dim(id, dimnamein.c_str(), sizein, NULL),__LINE__,__FILE__);
        }
      }
    };
    void print()
    {
      if(VERBOSE!=0)
      {
        cout<<name<<" (weight="<<weight<<") :"<<endl;
        if(VERBOSE==1)
        {
          cout<<"  "<<nbdim<<" dimension(s) and "<<nbvar<<" variable(s)"<<endl;
        }else{
          cout<<"  Dimension :"<<endl;
          for(int i=0; i<nbdim; i++) cout<<"    "<<dim[i].id<<") "<<dim[i].name<<" - "<<dim[i].size<<" element(s)"<<endl;
          
          cout<<"  Variable :"<<endl;
          for(int i=0; i<nbvar; i++)
          {
            cout<<"    "<<var[i].id<<") "<<var[i].name<<"(";
            for(int j=0; j<var[i].nbdim; j++)
            {
              if(j<var[i].nbdim-1)
              {
                cout<<dim[var[i].iddim[j]].name<<", ";
              }else{
                cout<<dim[var[i].iddim[j]].name;
              }
            }
            cout<<")"<<endl;
          }
        }
      }
    };
    bool isSimilar(fichier &file2)
    {
      if(nbdim!=file2.nbdim) return false;
      if(nbvar!=file2.nbvar) return false;
      if(idRecDim!=file2.idRecDim) return false;
      for(int i=0; i<nbvar; i++)
        if(var[i].name!=file2.var[i].name)
          return false;
        else if(var[i].nbdim!=file2.var[i].nbdim)
          return false;
        else{
          for(int j=0; j<var[i].nbdim; j++)
            if(var[i].iddim[j]!=file2.var[i].iddim[j])
              return false;
        }
      return true;
    };
    bool isAlsoDimension(int varnumber)
    {
      for(int i=0; i<nbdim; i++)
        if((dim[i].name==var[varnumber].name)||(dim[i].name+"_bounds"==var[varnumber].name))
          return true;
      return false;
    };
    void searchIdRecVar()
    {
      for(int i=0; i<nbvar; i++)
        if(dim[idRecDim].name==var[i].name)
          idRecVar=i;
    };
    int getVariableByName(string varname)
    {
      for(int i=0; i<nbvar; i++)
        if(var[i].name==varname)
          return i;
      return -1;
    }
    int getDimensionByName(string dimname)
    {
      for(int i=0; i<nbdim; i++)
        if(dim[i].name==dimname)
          return i;
      return -1;
    };
    
    void copyVar(variable source)
    {
      copyVar(source, source.name);
    };
    
    void copyVar(variable source, string targetname)
    {
      int dimtab[MAX_NC_DIMS];
      int nbVarDim=source.nbdim;
      for(int i=0; i<nbVarDim; i++) dimtab[i]=source.iddim[i];
      addvar(targetname, source.type, dimtab, nbVarDim);
    };
    
    void copyAttribute(fichier source, string varname)
    {
      copyAttribute(source, varname, varname);
    };
    
    void copyAttribute(fichier source, string varname, string targetname)
    {
      int idvarSource=source.getVariableByName(varname);
      int idvarCible=getVariableByName(targetname);
      
      int nbatt;
      char attname[MAX_NC_NAME];
      check(nc_inq_var(source.id, idvarSource, NULL, NULL, NULL, NULL, &nbatt),__LINE__,__FILE__);
      for(int i=0; i<nbatt; i++)
      {
        check(nc_inq_attname(source.id, idvarSource, i, attname),__LINE__,__FILE__);
        check(nc_copy_att(source.id, idvarSource, attname, id, idvarCible),__LINE__,__FILE__);
      }
    };
    
    void PutXaxisAtLon()
    {
      string name[5]={"lon", "Lon", "LON", "longitude", "Longitude"};
      int i=0;
      int idx=getVariableByName(name[0]);
      while(i<5 && idx==-1) idx=getVariableByName(name[i++]);
      if(idx!=-1) {
        int status=nc_inq_att(id, idx, "axis", NULL, NULL);
        if(status!=NC_NOERR) putAttribute(name[i], "axis", "X");
      }
    };
    
    void PutYaxisAtLon()
    {
      string name[5]={"lat", "Lat", "LAT", "latitude", "Latitude"};
      int i=0;
      int idx=getVariableByName(name[0]);
      while(i<5 && idx==-1) idx=getVariableByName(name[i++]);    
      if(idx!=-1) {
        int status=nc_inq_att(id, idx, "axis", NULL, NULL);
        if(status!=NC_NOERR) putAttribute(name[i], "axis", "Y");
      }
    };
    
    void getStructure()
    {
      int tempNbDim, tempNbVar;
      string RecName;
      check(nc_inq(id,  &tempNbDim,  &tempNbVar, NULL, &idRecDim),__LINE__,__FILE__);
    
      //Get the name for each input variable/dimension
      for(int i=0; i<tempNbDim; i++) //dimension
      {
        char dimname[MAX_NC_NAME];
        size_t size;
        check(nc_inq_dim(id, i, dimname, &size),__LINE__,__FILE__);
        if(i==idRecDim)
        {
          adddim(dimname, size, 1);
          RecName=dimname;
        }else
          adddim(dimname, size, 0);
      }
      for(int i=0; i<tempNbVar; i++) //variable
      {
        char varname[MAX_NC_NAME];
        nc_type type;
        int nbvardim;
        int dimidtab[MAX_NC_DIMS];
        check(nc_inq_var(id, i, varname, &type, NULL, NULL, NULL),__LINE__,__FILE__);
        check(nc_inq_var(id, i, NULL, NULL, &nbvardim, dimidtab, NULL),__LINE__,__FILE__);
        if(RecName==varname) idRecVar=i;
        addvar(varname, type, dimidtab, nbvardim);
      }
    };
    
    void putAttribute(string varname, string attribute, string value)
    {
      int idvar=getVariableByName(varname);
      check(nc_put_att(id, idvar, attribute.c_str(), NC_CHAR, value.size(), value.c_str()),__LINE__,__FILE__);
    };
};

bool is_readable(const string &file) 
{ 
    ifstream fichier(file.c_str()); 
    return !fichier.fail(); 
}

void AccumulateMean(double *out, void *in, double w, long size, nc_type type)
{
  short *inS;
  int *inI;
  float *inF;
  double *inD;
  switch(type)
  {
    case(3):
      inS=(short*)in;
      for(long i=0; i<size; i++)out[i]+=(double)(inS[i])*w;
      break;
    case(4):
      inI=(int*)in;
      for(long i=0; i<size; i++)out[i]+=(double)(inI[i])*w;
      break;
    case(5):
      inF=(float*)in;
      for(long i=0; i<size; i++)out[i]+=(double)(inF[i])*w;
      break;
    case(6):
      inD=(double*)in;
      for(long i=0; i<size; i++)out[i]+=(double)(inD[i])*w;
      break;
    default:
      cout<<"Unknown type !!!"<<endl;
  }
}

void addCte(void *table, void *cte, long size, nc_type type)
{
  short *tableS;
  int *tableI;
  float *tableF;
  double *tableD;
  switch(type)
  {
    case(3):
      tableS=(short*)table;
      for(long i=0; i<size; i++)tableS[i]+=((short*)cte)[0];
      break;
    case(4):
      tableI=(int*)table;
      for(long i=0; i<size; i++)tableI[i]+=((int*)cte)[0];
      break;
    case(5):
      tableF=(float*)table;
      for(long i=0; i<size; i++)tableF[i]+=((float*)cte)[0];
      break;
    case(6):
      tableD=(double*)table;
      for(long i=0; i<size; i++)tableD[i]+=((double*)cte)[0];
      break;
    default:
      cout<<"Unknown type !!!"<<endl;
  }
}
void ConvertfromDouble(void *out, double *in, long size, nc_type type)
{
  short *outS;
  int *outI;
  float *outF;
  double *outD;
  switch(type)
  {
    case(3):
      outS=(short*)out;
      for(long i=0; i<size; i++)outS[i]=(short)(round(in[i]));
      break;
    case(4):
      outI=(int*)out;
      for(long i=0; i<size; i++)outI[i]=(int)(round(in[i]));
      break;
    case(5):
      outF=(float*)out;
      for(long i=0; i<size; i++)outF[i]=(float)(in[i]);
      break;
    case(6):
      outD=(double*)out;
      for(long i=0; i<size; i++)outD[i]=in[i];
      break;
    default:
      cout<<"Unknown type !!!"<<endl;
  }
}

int main(int argc,char **argv)
{
  //Check execution argument
  if((argc<4)||(argc%2==0))
  {
    cout<<"Usage :"<<endl;
    cout<<"\tNetCDFensStore [inputNetCDF] [Weight]...[inputNetCDF] [Weight] [outputFile] [Variable_name]"<<endl;
    return 1;
  }
  
  
  /////////////////////////////
  ////  INPUT DEFINITION   ////
  /////////////////////////////
  
  //open the input file
  vector <fichier> filein;
  
  string variablename=argv[argc-1];
  string namemean=variablename; namemean+="_mean";
    
  //read arguments
  for(int i=1; i<argc-2; i+=2) filein.push_back(fichier(argv[i],atoi(argv[i+1])));

  //loop on input files
  for(unsigned int numfile=0; numfile<filein.size(); numfile++)
  {
    //open the file
    cout<<"Open "<<filein[numfile].name<<endl;
    filein[numfile].open(NC_NOWRITE);
      
    //Get the number of variables and dimensions in the file
    cout<<"Read structure of "<<filein[numfile].name<<endl;
    filein[numfile].getStructure();
    
    //filein[numfile].print();
  
    //Check dimension and variable are the same (size, name, number) for all input file
    if(numfile>=1)
    {
      if(!filein[numfile-1].isSimilar(filein[numfile]))
      {
        cout<<filein[numfile-1].name<<" and "<<filein[numfile].name<<" have differente structur"<<endl;
        return 1;
      }
    }
  }
  
  
  /////////////////////////////
  ////  OUTPUT DEFINITION  ////
  /////////////////////////////
  
    
  //open/create the output file
  fichier fileout(argv[argc-2],1);
  bool isappend=is_readable(fileout.name);

  
  if(isappend)
  {
    cout<<"Append the "<<fileout.name<<" file"<<endl;
    fileout.open(NC_WRITE);
    fileout.getStructure();
    
  }else{
    fileout.create(NC_NETCDF4|NC_CLOBBER);
    //insert dimension of input file in the structure of output file
    for(int dimid=0; dimid<filein[0].nbdim; dimid++)
    {
      cout<<"copy dim "<<filein[0].dim[dimid].name<<endl;
      if(filein[0].idRecDim==dimid)
        fileout.adddim(filein[0].dim[dimid].name, 0, 1);
      else
        fileout.adddim(filein[0].dim[dimid].name, filein[0].dim[dimid].size, 0);
    }
    //add the new dimension "ens"
    fileout.adddim("ens", (int)filein.size(), 0);
    
    //insert dimension of input file in the structure of output file
    for(int varid=0; varid<filein[0].nbvar; varid++)
    {
      if(filein[0].isAlsoDimension(varid))
      {
        fileout.copyVar(filein[0].var[varid]);
      }else if(filein[0].var[varid].name=="tlat")
      {
        fileout.copyVar(filein[0].var[varid]);
      }else if(filein[0].var[varid].name=="tlon")
      {
        fileout.copyVar(filein[0].var[varid]);
      }else if(variablename==filein[0].var[varid].name)
      {
        int dimtab[MAX_NC_DIMS];
        for(int i=0; i<filein[0].var[varid].nbdim; i++) dimtab[i]=filein[0].var[varid].iddim[i];
        dimtab[filein[0].var[varid].nbdim]=fileout.nbdim-1; //the last dim (ens) is the last dimension for this variable
        fileout.addvar(filein[0].var[varid].name, filein[0].var[varid].type, dimtab, filein[0].var[varid].nbdim+1);
      }
      fileout.copyAttribute(filein[0],filein[0].var[varid].name); 
    }
    //add the variable corresponding to the dim "ens"
    int dimtab[MAX_NC_DIMS];
    dimtab[0]=fileout.getDimensionByName("ens");
    fileout.addvar("ens", NC_INT, dimtab, 1);
    fileout.putAttribute("ens", "long_name", "Ensemble");
    fileout.putAttribute("ens", "standard_name", "ens");
    fileout.putAttribute("ens", "axis", "E");
    
    //add the variable weight
    dimtab[0]=fileout.getDimensionByName(fileout.dim[fileout.idRecDim].name);
    dimtab[1]=fileout.getDimensionByName("ens");
    fileout.addvar("weight", NC_INT, dimtab, 2);
    fileout.putAttribute("weight", "long_name", "Weight Ensemble");
    fileout.putAttribute("weight", "standard_name", "weight");
    
    //add the mean variable
    fileout.copyVar(filein[0].var[filein[0].getVariableByName(variablename)], namemean);
    fileout.copyAttribute(filein[0],filein[0].var[filein[0].getVariableByName(variablename)].name, namemean); 
    
    //add missing argument for grads
    fileout.PutXaxisAtLon();
    fileout.PutYaxisAtLon();
    
    //editing structure is finish and can be write now
    fileout.WriteSturcture();
    fileout.searchIdRecVar();
  }

  
  
  //calc
  cout<<"Put data in "<<fileout.name<<endl;
  for(int idxvar=0; idxvar<fileout.nbvar; idxvar++)
  {
    int idxvarin=filein[0].getVariableByName(fileout.var[idxvar].name);
    
    if(idxvar==fileout.idRecVar)
    {
      cout<<fileout.var[idxvar].name<<" is the record dimension (append if needed)"<<endl;
      size_t start[1]={0};
      size_t count[1]={filein[0].var[idxvarin].size};
      void *value = (void *)malloc(count[0]*nctypelen(fileout.var[idxvar].type));
      check(nc_get_vara(filein[0].id, filein[0].var[idxvarin].id, start, count, value),__LINE__,__FILE__);
      
      if(isappend)
      {
        start[0]=fileout.var[idxvar].size-1;
        count[0]=1;
        void *valueLast = (void *)malloc(nctypelen(fileout.var[idxvar].type));
        check(nc_put_vara(fileout.id, fileout.var[idxvar].id, start, count, valueLast),__LINE__,__FILE__);
        count[0]=filein[0].var[idxvarin].size;
        addCte(value, valueLast, count[0], fileout.var[idxvar].type);
        free(valueLast);
      }
      
      start[0]=fileout.var[idxvar].size;
      count[0]=filein[0].var[idxvarin].size;
      check(nc_put_vara(fileout.id, fileout.var[idxvar].id, start, count, value),__LINE__,__FILE__);
      free(value);
    }else if(fileout.var[idxvar].name==variablename)
    {
      cout<<fileout.var[idxvar].name<<" is merge by ensemble"<<endl; 
      size_t startout[fileout.var[idxvar].nbdim];
      size_t countout[fileout.var[idxvar].nbdim];
      long totalsizeout=1;
      for(int k=0; k<fileout.var[idxvar].nbdim; k++)
      {
        startout[k]=0;
        //countout[k]=filein[0].dim[fileout.var[idxvar].iddim[k]].size;
        countout[k]=fileout.dim[fileout.var[idxvar].iddim[k]].size;
        if(filein[0].idRecDim==fileout.var[idxvar].iddim[k]) countout[k]=filein[0].dim[fileout.var[idxvar].iddim[k]].size;
        totalsizeout*=countout[k];
      }
      startout[0]=fileout.var[fileout.idRecVar].size;
      size_t startin[filein[0].var[idxvarin].nbdim];
      size_t countin[filein[0].var[idxvarin].nbdim];
      long totalsizein=1;
      for(int k=0; k<filein[0].var[idxvarin].nbdim; k++)
      {
        startin[k]=0;
        countin[k]=filein[0].dim[filein[0].var[idxvarin].iddim[k]].size;
        totalsizein*=countin[k];
      }
      void *value = (void *)malloc(totalsizein*nctypelen(fileout.var[idxvar].type)); //contain input variable
      void *valueTot = (void *)malloc(totalsizeout*nctypelen(fileout.var[idxvar].type)); //contain output where input variable are concatenate in the E dimension
    
      for(unsigned int dimE=0; dimE<filein.size(); dimE++)
      {
        check(nc_get_vara(filein[dimE].id, filein[dimE].var[idxvarin].id, startin, countin, value),__LINE__,__FILE__);
        if(fileout.var[idxvar].type!=6)
        {
          for(int i=0; i<totalsizein; i++) ((float*)valueTot)[dimE+filein.size()*i]=((float*)value)[i];
        }else{
          for(int i=0; i<totalsizein; i++) ((double*)valueTot)[dimE+filein.size()*i]=((double*)value)[i];
        }
      }
      check(nc_put_vara(fileout.id, idxvar, startout, countout, valueTot),__LINE__,__FILE__);
      free(value);
      free(valueTot);
    }else if(!isappend && fileout.var[idxvar].name!="ens" && fileout.var[idxvar].name!="weight" && fileout.var[idxvar].name!=namemean)
    {
      cout<<fileout.var[idxvar].name<<" is copied"<<endl;

      size_t start[fileout.var[idxvar].nbdim];
      size_t count[fileout.var[idxvar].nbdim];
      long totalsize=1;
      for(int k=0; k<fileout.var[idxvar].nbdim; k++)
      {
        start[k]=0;
        count[k]=fileout.dim[fileout.var[idxvar].iddim[k]].size;
        if(fileout.idRecDim==fileout.var[idxvar].iddim[k]) count[k]=filein[0].dim[fileout.var[idxvar].iddim[k]].size;
        totalsize*=count[k];
      }
      start[0]=fileout.var[fileout.idRecVar].size;
      
      void *value = (void *)malloc(totalsize*nctypelen(fileout.var[idxvar].type));
      check(nc_get_vara(filein[0].id, filein[0].var[idxvarin].id, start, count, value),__LINE__,__FILE__);
      check(nc_put_vara(fileout.id, fileout.var[idxvar].id, start, count, value),__LINE__,__FILE__);
      free(value);
    }
  }
  
  if(!isappend)
  {
    //Put ens value
    void *value = (void *)malloc(filein.size()*nctypelen(NC_INT));
    for(unsigned int dimE=0; dimE<filein.size(); dimE++) ((int*)value)[dimE]=dimE;
    size_t start[1]={0};
    size_t count[1]={filein.size()};
    int idxens=fileout.getVariableByName("ens");
    check(nc_put_vara(fileout.id, idxens, start, count, value),__LINE__,__FILE__);
    //free(value);
  }
  
  //Put Weight value
  long fileouttimesize=filein[0].var[filein[0].idRecVar].size;
  void *value = (void *)malloc(fileouttimesize*filein.size()*nctypelen(NC_INT));
  for(unsigned int dimE=0; dimE<filein.size(); dimE++)
    for(unsigned int t=0; t<fileouttimesize; t++)
      ((int*)value)[dimE+filein.size()*t]=(int)filein[dimE].weight;
  size_t start[2]={fileout.var[fileout.idRecVar].size, 0};
  size_t count[2]={fileouttimesize, filein.size()};
  int idxweight=fileout.getVariableByName("weight");
  check(nc_put_vara(fileout.id, idxweight, start, count, value),__LINE__,__FILE__);
  free(value);
  
  //calc and put mean value
  int idxvar=fileout.getVariableByName(namemean);
  int idxvarin=filein[0].getVariableByName(variablename);
  size_t startmean[filein[0].var[idxvarin].nbdim];
  size_t countmean[filein[0].var[idxvarin].nbdim];
  size_t steptime=filein[0].dim[filein[0].var[idxvarin].iddim[0]].size/12;
  //for(int t=0; t<filein[0].dim[filein[0].var[idxvarin].iddim[0]].size; t+=steptime)
  //{
    long totalsize=1;
    for(int k=0; k<filein[0].var[idxvarin].nbdim; k++)
    {
     // if(k==0)
     // {
      //  startmean[k]=t;
      //  countmean[k]=steptime;
     // }else{
        startmean[k]=0;
        countmean[k]=filein[0].dim[filein[0].var[idxvarin].iddim[k]].size;
     // }
      totalsize*=countmean[k];
    }
    void *valuein = (void *)malloc(totalsize*nctypelen(filein[0].var[idxvarin].type));
    double *valuecalc = (double *)malloc(totalsize*sizeof(double));
    
    double sumweight=0.;
    for(unsigned int i=0; i<filein.size(); i++)
      sumweight+=filein[i].weight;
    
    for(unsigned int i=0; i<filein.size(); i++)
    {
      check(nc_get_vara(filein[i].id, filein[i].var[idxvarin].id, startmean, countmean, valuein),__LINE__,__FILE__);
      AccumulateMean(valuecalc, valuein, filein[i].weight/sumweight, totalsize, filein[0].var[idxvarin].type);
    }
    free(valuein);
    void *valueout = (void *)malloc(totalsize*nctypelen(filein[0].var[idxvarin].type));
    ConvertfromDouble(valueout, valuecalc, totalsize, filein[0].var[idxvarin].type);
    free(valuecalc);
    startmean[0]=fileout.var[fileout.idRecVar].size;
    check(nc_put_vara(fileout.id, fileout.var[idxvar].id, startmean, countmean, valueout),__LINE__,__FILE__);
    free(valueout);
  //}
  
  //Close input netCDF files
  for(unsigned int numfile=0; numfile<filein.size(); numfile++) ncclose(filein[numfile].id);
  check(nc_close(fileout.id),__LINE__,__FILE__);
  cout<<fileout.name<<" is writed"<<endl; 

  return 0;
}
