// NetCDFwavg -- NetCDF weighted average 
// is based on ncfint 1.29 developped under GPL licence by
//    Charlie Zender
//    Department of Earth System Science
//    University of California at Irvine
//    Irvine, CA 92697-3100

#include <math.h>
#include <string>
#include <iostream>
#include <vector>
#include <netcdf.h>

using namespace std;

class variable
{
  public:
    string name;
    int id;
    int nbdim;
    long size;
    nc_type type;
    vector<int> iddim;
    variable(string namein, int idin) { id=idin; name=namein; nbdim=0; size=1; totaltime=1; };
    void adddim(int idin, long sizein) { iddim.push_back(idin); nbdim++; size*= sizein; };
    int totaltime;
};

class fichier
{
  public:
    string name;
    double weight;
    int id;
    vector<variable> var;
    vector<variable> dim;
    int nbvar;
    int nbdim;
    int idRecDim;
    
    fichier(string namein) { name=namein; weight=1.; nbvar=0; nbdim=0; idRecDim=-1; };
    fichier(string namein, double weightin) { name=namein; weight=weightin; nbvar=0; nbdim=0; idRecDim=-1; };
    void addvar(string varnamein, int idin, nc_type typein) { var.push_back(variable(varnamein, idin)); var[var.size()-1].type=typein; nbvar++; };
    void adddim(string dimnamein, int idin, long sizein) { dim.push_back(variable(dimnamein, idin)); dim[dim.size()-1].size=sizein; nbdim++; };
    void print()
    {
	cout<<name<<" (weight normalized="<<weight<<")"<<endl;
    };
};


void AccumulateMean(double *out, void *in, double w, long size, nc_type type)
{
  short *inS;
  int *inI;
  float *inF;
  double *inD;
  switch(type)
  {
    case(3):
      inS=(short*)in;
      for(long i=0; i<size; i++)out[i]+=(double)(inS[i])*w;
      break;
    case(4):
      inI=(int*)in;
      for(long i=0; i<size; i++)out[i]+=(double)(inI[i])*w;
      break;
    case(5):
      inF=(float*)in;
      for(long i=0; i<size; i++)out[i]+=(double)(inF[i])*w;
      break;
    case(6):
      inD=(double*)in;
      for(long i=0; i<size; i++)out[i]+=(double)(inD[i])*w;
      break;
    default:
      cout<<"Unknown type !!!"<<endl;
  }
}

void ConvertfromDouble(void *out, double *in, long size, nc_type type)
{
  short *outS;
  int *outI;
  float *outF;
  double *outD;
  switch(type)
  {
    case(3):
      outS=(short*)out;
      for(long i=0; i<size; i++)outS[i]=(short)(round(in[i]));
      break;
    case(4):
      outI=(int*)out;
      for(long i=0; i<size; i++)outI[i]=(int)(round(in[i]));
      break;
    case(5):
      outF=(float*)out;
      for(long i=0; i<size; i++)outF[i]=in[i];
      break;
    case(6):
      outD=(double*)out;
      for(long i=0; i<size; i++)outD[i]=in[i];
      break;
    default:
      cout<<"Unknown type !!!"<<endl;
  }
}

int main(int argc,char **argv)
{
  vector<fichier> file;
  if(argc%2!=0 || argc<4)
  {
    cout<<"Usage :"<<endl;
    cout<<"\tNetCDFwavg input1.nc weight1 input2.nc weight2 ... output.nc"<<endl;
    exit(1);
  }
  double totalweight=0;
  for(int i=1; i<argc-1; i+=2)
  {
    file.push_back(fichier(argv[i],atof(argv[i+1])));
    totalweight+=atof(argv[i+1]);
  }
  for(unsigned int i=0; i<file.size(); i++)
  {
    file[i].weight/=totalweight;
  }


  
  //loop on input files
  for(unsigned int i=0; i<file.size(); i++)
  {    
    //open the file
    file[i].id=ncopen(file[i].name.c_str(),NC_NOWRITE);
    
    //Get the number of variables and dimensions in the file
    int nbdim;
    int nbvar;
    ncinquire(file[i].id, &nbdim, &nbvar, NULL, &file[i].idRecDim);
    

    //Form initial extraction list from user input
    for(int j=0; j<nbvar; j++)
    {
      //Get the name for each variable.
      char varname[MAX_NC_NAME];
      nc_type type;
      ncvarinq(file[i].id, j, varname, &type, NULL, NULL, NULL);
      file[i].addvar(varname, j, type);
    }
     
    //Add all coordinate variables to extraction list
    for(int j=0; j<nbdim; j++)
    {
      char dimname[MAX_NC_NAME];
      long size;
      ncdiminq(file[i].id, j, dimname, &size);
      file[i].adddim(dimname, j, size);
    }
      

    //Get the number of dimensions
    for(int j=0; j<file[i].nbvar; j++)
    {
      int nbvardim;
      int dimidtab[MAX_NC_DIMS];
      ncvarinq(file[i].id, file[i].var[j].id, NULL, NULL, &nbvardim, dimidtab, NULL);
      for(int k=0; k<nbvardim; k++) file[i].var[j].adddim(dimidtab[k], file[i].dim[dimidtab[k]].size);
    }
    file[i].print();
  }
  
  //create output filename
  /*NcFile dataFile("simple_xy.nc", NcFile::Replace);
  if (!dataFile.is_valid())
  {
    cout << "Couldn't open file!\n";
    return NC_ERR;
  }
  NcDim* xDim = dataFile.add_dim("x", NX);
  NcDim* yDim = dataFile.add_dim("y", NY);
  NcVar *data = dataFile.add_var("data", ncInt, xDim, yDim);
  */
  
  //Open output file
  int outid=nccreate(argv[argc-1], NC_CLOBBER);
    
  //Copy global attributes
  int nbatt;
  ncinquire(file[0].id, NULL, NULL, &nbatt, NULL);  
  for(int i=0; i<nbatt; i++)
  {
    char attname[MAX_NC_NAME];
    ncattname(file[0].id, NC_GLOBAL, i, attname);
    ncattcopy(file[0].id, NC_GLOBAL, attname, outid, NC_GLOBAL);
  }
    
  //Define dimensions in output file
  int totaltime;
  for(int i=0; i<file[0].nbdim; i++)
  {
    if(file[0].idRecDim==file[0].dim[i].id)
    {
      ncdimdef(outid, file[0].dim[i].name.c_str(), NC_UNLIMITED);
      totaltime=file[0].dim[i].size;
    }else{
      ncdimdef(outid, file[0].dim[i].name.c_str(), file[0].dim[i].size);
    }
  }
  
  //Define variables in output file, and copy their attributes
  for(int i=0; i<file[0].nbvar; i++)
  {
    int dimtab[MAX_NC_DIMS];
    for(int j=0; j<file[0].var[i].nbdim; j++)
    {
      dimtab[j]=file[0].var[i].iddim[j];
      if(file[0].var[i].iddim[j]==file[0].idRecDim) file[0].var[i].totaltime=totaltime;
    }
    int outvarid=ncvardef(outid, file[0].var[i].name.c_str(), file[0].var[i].type, file[0].var[i].nbdim, dimtab);

    int nbatt;
    ncvarinq(file[0].id, i, NULL, NULL, NULL, NULL, &nbatt);
    for(int j=0; j<nbatt; j++)
    {
      char attname[MAX_NC_NAME];
      ncattname(file[0].id, file[0].var[i].id, j, attname);
      ncattcopy(file[0].id, file[0].var[i].id, attname, outid, outvarid);
    }
  }
  
  //Turn off default filling behavior to enhance efficiency
  ncsetfill(outid, NC_NOFILL);
  
  //Take output file out of define mode
  ncendef(outid);

  //calc
  cout<<"#";for(int j=0; j<file[0].nbvar-2; j++)cout<<"-";cout<<"#"<<endl;
  for(int j=0; j<file[0].nbvar; j++)
  {
    int step=1;
    if(file[0].var[j].totaltime>1)
    {
      if(file[0].var[j].totaltime%120==0)
      {
        step=120;
      }else if(file[0].var[j].totaltime%12==0)
      {
        step=12;
      }else if(file[0].var[j].totaltime%115==0)
      {
        step=115;
      }
    }
    cout<<"#"<<flush;
    for(int t=0; t<totaltime; t+=step)
    {
      long start[file[0].var[j].nbdim];
      long count[file[0].var[j].nbdim];
      long totalsize=1;
      for(int k=0; k<file[0].var[j].nbdim; k++)
      {
        if(file[0].idRecDim==file[0].var[j].iddim[k])
        {
          start[k]=t;
          count[k]=step;
        }else{
          start[k]=0;
          count[k]=file[0].dim[file[0].var[j].iddim[k]].size;        
        }
        totalsize*=count[k];
      }
      void *valuein = (void *)malloc(totalsize*nctypelen(file[0].var[j].type));
      double *valuecalc = (double *)malloc(totalsize*sizeof(double));
      void *valueout = (void *)malloc(totalsize*nctypelen(file[0].var[j].type));
      
      for(unsigned int i=0; i<file.size(); i++)
      {
        ncvarget(file[i].id, file[i].var[j].id, start, count, valuein);
        AccumulateMean(valuecalc, valuein, file[i].weight, totalsize, file[0].var[j].type);
      }
      ConvertfromDouble(valueout, valuecalc, totalsize, file[0].var[j].type);
      ncvarput(outid, file[0].var[j].id, start, count, valueout);
      free(valuein); free(valuecalc); free(valueout);
    }
  }
  cout<<endl;
  cout<<"==> Avg file "<<argv[argc-1]<<" created"<<endl;
    
  //Close input netCDF files
  for(unsigned int i=0; i<file.size(); i++) ncclose(file[i].id);
  ncclose(outid);  

  return 0;
}
